/*
 * This file is part of AEGIS
 *
 * Copyright (C) 2009 Nokia Corporation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 * Author: Markku Savela
 */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <err.h>
#include "librestok.h"
#include "status_cnf.h"

// Main progam for a class of fixup tasks that
// restok.conf file might need.

int main(int argc, char **argv)
	{
	const char *config = NULL;
	const char *origin = NULL;
	const char *arg;
	// Process the command line
	while ((arg = *++argv) != NULL && --argc > 0)
		{
		int option;

		if (*arg != '-')
			break;
		++arg;
		while ((option = *arg++) != 0)
			{
			switch (option)
				{
				case 'c':
					// Override the default configuration
					config = arg;
					if (*config == 0 && argc > 1)
						{
						config = *++argv;
						--argc;
						}
					goto next_arg;

				case 's':
					// Set default origin
					origin = arg;
					if (*origin == 0 && argc > 1)
						{
						origin = *++argv;
						--argc;
						}
					goto next_arg;

				default:
					warnx("Unkonwn option '%c'\n", option);
					warnx("Usage\n\taegis-fixup [-s defaultorigin]\n");
					exit(EXIT_FAILURE);
				}
			}
		next_arg:
		;
		}

	// Load the configuration into memory.
	struct RESTOK::configuration *cnf =
		(struct RESTOK::configuration *)restok_cnf_load(config, NULL, 0);
	if (cnf == NULL)
		{
		warnx("Could not load the configuration -- nothing can do!");
		// Exit with failure only, if configuration file was
		// explicitly defined (and not present).
		exit(config ? EXIT_FAILURE : EXIT_SUCCESS);
		}

	// For now, the 'fixup' only hash single action: read through dpkg status
	// information and for package mentioned there:
	// - if package is not installed => remove entry from restok.conf
	// - if package is installed => make entry into restok.conf (if not already there)
	// Additionally, if the origin is specified, then mark this origin for each
	// installed package in restok.conf, which does not yet have an origin.

	struct STATUS::configuration *status = STATUS::configuration::merge(origin, cnf);
	if (status == NULL)
		{
		warnx("Could not merge dpkg status -- nothing can do!");
		// Exit with failure only, if configuration file was
		// explicitly defined (and not present).
		restok_cnf_free(cnf);
		exit(EXIT_FAILURE);
		}

	restok_cnf_dump(cnf);

	// Release the configuration data.
	status->release();
	exit(EXIT_SUCCESS);
	}
