/*
 * This file is part of AEGIS
 *
 * Copyright (C) 2009-2010 Nokia Corporation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 * Author: Markku Savela
 */

#ifdef HAVE_LINUX_AEGIS_RESTOK_H

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "librestok.h"
#include <linux/aegis/restokctl.h>
#include <linux/aegis/restok.h>

static const char *const tokens_file = "/sys/kernel/security/restok/tokens";

static long doit(int op, long id, size_t len, char* name)
	{
	struct restok_ioc_arg arg = { .id = id, .len = len, .name = name };
	long result;

	int fd = open(tokens_file, O_RDONLY);
	if (fd == -1)
		return -1;

	result = ioctl(fd, op, &arg);
	close(fd);
	return result;
	}

long restok_locate(const char* name)
	{
	return name ? doit(SIOCRESTOK_LOCATE, 0, strlen(name), (char *)name) : -1;
	}

long restok_define(const char* name, long owner)
	{
	return name ? doit(SIOCRESTOK_DEFINE, owner, strlen(name), (char *)name) : -1;
	}

int restok_string(long id, char *str, size_t str_len)
	{
	return doit(SIOCRESTOK_STRING, id, str_len, str);
	}

#ifndef RESTOK_RESOURCE_MANAGER
int restok_setlock(long id)
	{
	return doit(SIOCRESTOK_SETLOCK, id, 0, NULL);
	}
#endif

#else
#include "restok_fallback.h"
#include <errno.h>
#include <err.h>

long restok_locate(const char* name)
	{
	warnx("restok_locate not available -- librestok not compiled with kernel headers");
	errno = EPERM;
	return -1;
	}

long restok_define(const char* name, long owner)
	{
	warnx("restok_define not available -- librestok not compiled with kernel headers");
	errno = EPERM;
	return -1;
	}

int restok_string(long id, char *str, size_t str_len)
	{
	/* Not supported for now */
	warnx("restok_string not available -- librestok not compiled with kernel headers");
	return 0;
	}

int restok_setlock(long id)
	{
	/* Not supported for now */
	warnx("restok_setlock not available -- librestok not compiled with kernel headers");
	return 0;
	}
#endif
