/*
 * This file is part of AEGIS
 *
 * Copyright (C) 2009 Nokia Corporation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 * Author: Markku Savela
 */
#ifndef RESTOK_DEF_H
#define RESTOK_DEF_H

/*!
 * @file restok_def.h
 *
 * This is header only (no .cpp or .c) and defines some
 * generally used macroes for debugging error reporting
 * used within librestok library.
 *
 * The current implementation uses the <err.h> service.
 *
 * @package librestok
 */
#include <err.h>

#undef DBG
#undef DBGG
#undef ERR
#undef ERRX

/*!
 * Enclose a sequence of statements for debugging only.
 *
 * The argument list is a sequence of statements.
 */
#ifdef RESTOK_DEBUG
#	define DBG(S...) do { S; } while (0)
#else
#	define DBG(S...)
#endif

/*!
 * Easy way to selectively disable some debugging
 * statements, even while debug enabled.
 *
 * The argument list is a sequence of statements.
 */
#define DBGG(S...)

/*!
 * Unconditional error message with errno.
 *
 * The 'args..' is a printf format and arguments.
 */
#define ERR(args...) warn(args)

/*!
 * Unconditional error message without errno.
 *
 * The 'args..' is a printf format and arguments.
 */
#define ERRX(args...) warnx(args)

#endif
