/*
 * This file is part of AEGIS
 *
 * Copyright (C) 2010 Nokia Corporation
 *
 * Contact: Laszlo Papp <ext-laszlo.papp@nokia.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 * Author: Laszlo Papp
 */
#ifndef _SOCKETPOLICY_H_
#define _SOCKETPOLICY_H_

#include <features.h>
__BEGIN_DECLS

struct skt_ioc_args;

/**
* socket_policy_start()
*
* This function is to be called to allocate a default memory area for the socket policies
*
* Notes
* Return a pointer to the allocated memory. On error, these functions return NULL.
*/
struct skt_ioc_args *socket_policy_start();

/**
 * socket_policy_add() - Add a new socket policy that will be loaded into the kernel
 * @handle:   contains the handle structure for the socket and it will be modified
 * @socketstr: contains the socket family, type and protocol values in one string with separator (' ')
 * @cred_type: contains the credential the required type
 * @cred_value: contains the credential the required value
 *
 * Return 0 if the socket and credential parameters could be added successfully
 */
int socket_policy_add(struct skt_ioc_args *handle, const char *socketstr, int cred_type, int cred_value);

/**
 * socket_policy_finish() - Finish the socket policy addition session
 * @handle:    contains the handle structure for the socket that is loaded into the kernel
 *
 * Return 0 if the socket policy could be loaded into the kernel module
 */
int socket_policy_finish(struct skt_ioc_args *handle);

/**
 * socket_policy_abort() - Abort the socket policy addition or starting session
 * @handle:    contains the handle structure for the socket that will be deallocated
 *
 * Return 0 if the socket policy could be loaded into the kernel module
 */
void socket_policy_abort(struct skt_ioc_args *handle);

__END_DECLS

#endif
